@@ -1,6 +1,6 @@
 /*
  *  This file is part of pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2017 David J. Barnes
+ *  Copyright (C) 1994-2018 David J. Barnes
  *
  *  pgn-extract is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -47,7 +47,7 @@
 #define DEFAULT_POSITIONAL_DEPTH 300
 
 /* Prototypes of functions limited to this file. */
-static Boolean position_matches(const Board *board);
+static const char *position_matches(const Board *board);
 static Boolean play_moves(Game *game_details, Board *board, Move *moves,
         unsigned max_depth, Boolean check_move_validity,
         Boolean mainline);
@@ -56,6 +56,7 @@ static Boolean apply_variations(const Game *game_details, const Board *board,
 static Boolean rewrite_variations(const Board *board, Variation *variation);
 static Boolean rewrite_moves(Game *game, Board *board, Move *move_details);
 static void build_FEN_components(const Board *board, char *epd, char *fen_suffix);
+static Boolean drop_plys(Game *game, Move *moves, unsigned plys_to_drop);
 #if 0
 static void append_evaluation(Move *move_details, const Board *board);
 static void append_FEN_comment(Move *move_details, const Board *board);
@@ -64,6 +65,8 @@ static void append_hashcode_comment(Move *move_details, Board *board);
 static double evaluate(const Board *board);
 static double shannonEvaluation(const Board *board);
 static void print_board(const Board *board, FILE *outfp);
+static StringList * find_matching_comment(const char *comment_pattern,
+                                          const CommentList *comment);
 
 /* The English SAN piece characters. These are
  * always used when building a FEN string, rather
@@ -157,7 +160,7 @@ copy_string(const char *str)
     char *result;
     size_t len = strlen(str);
 
-    result = malloc_or_die(len + 1);
+    result = (char *) malloc_or_die(len + 1);
     strcpy(result, str);
     return result;
 }
@@ -620,7 +623,7 @@ new_fen_board(const char *fen)
 Board *
 new_game_board(const char *fen)
 {
-    Board *new_board;
+    Board *new_board = NULL;
     static const Board initial_board ={
         {
             { OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF},
@@ -906,12 +909,14 @@ play_moves(Game *game_details, Board *board, Move *moves, unsigned max_depth,
     /* Whether there has been an underpromotion. */
     Boolean underpromotion = FALSE;
     
+    const char *match_label = NULL;
+    
     /* Try the initial board position for a match.
      * This is required because the game might have been set up
      * from a FEN string, rather than being the normal starting
      * position.
      */
-    if (!game_matches && position_matches(board)) {
+    if (!game_matches && (match_label = position_matches(board)) != NULL) {
         game_matches = TRUE;
         if (GlobalState.add_position_match_comments) {
             CommentList *comment = create_match_comment(board);
@@ -946,7 +951,7 @@ play_moves(Game *game_details, Board *board, Move *moves, unsigned max_depth,
             if (check_move_validity) {
                 if (apply_move(next_move, board)) {
                     /* Don't try for a positional match if we already have one. */
-                    if (!game_matches && position_matches(board)) {
+                    if (!game_matches && (match_label = position_matches(board)) != NULL) {
                         game_matches = TRUE;
                         if (GlobalState.add_position_match_comments) {
                             CommentList *comment = create_match_comment(board);
@@ -1060,19 +1065,37 @@ play_moves(Game *game_details, Board *board, Move *moves, unsigned max_depth,
                         }
 
 
-                        /* Check for inconsistentcy between Result tag and game result. */
+                        /* Check for inconsistency between Result tag and game result. */
                         const char *move_result = next_move->terminating_result;
                         const char *result_tag = game_details->tags[RESULT_TAG];
-                        if(result_tag != NULL &&
-                               move_result != NULL &&
-                               strcmp(move_result, "*") != 0) {
-                            if(strcmp(result_tag, move_result) != 0) {
+                        if (result_tag != NULL && move_result != NULL &&
+                                strcmp(result_tag, move_result) != 0) {
+                            // Mismatch.
+                            // Whether to report it.
+                            Boolean report = TRUE;
+                            if (GlobalState.fix_result_tags) {
+                                if(strcmp(move_result, "*") == 0 || 
+                                        strcmp(result_tag, "*") == 0) {
+                                    // Prefer the move result.
+                                    free((void *) result_tag);
+                                    game_details->tags[RESULT_TAG] = copy_string(move_result);
+                                    report = FALSE;
+                                }
+                                else {
+                                    // Irreconcilable conflict.
+                                }
+                            }
+                            else if(strcmp(move_result, "*") == 0) {
+                                // Don't report this form of inconsistency.
+                                report = FALSE;
+                            }
+                            if(report) {
                                 print_error_context(GlobalState.logfile);
                                 fprintf(GlobalState.logfile,
-                                        "Inconsistent result strings %s vs %s in the following game.\n", 
+                                        "Inconsistent result strings %s vs %s in the following game.\n",
                                         result_tag, move_result);
                                 report_details(GlobalState.logfile);
-                                if(GlobalState.reject_inconsistent_results) {
+                                if (GlobalState.reject_inconsistent_results) {
                                     game_ok = FALSE;
                                 }
                             }
@@ -1208,6 +1231,13 @@ play_moves(Game *game_details, Board *board, Move *moves, unsigned max_depth,
         if(GlobalState.match_underpromotion && !underpromotion) {
             game_matches = FALSE;
         }
+        
+        /* Add a tag containing the matching FENPattern  label
+         * if appropriate.
+         */
+        if(GlobalState.add_matchlabel_tag && match_label != NULL && *match_label != '\0') {
+            game_details->tags[MATCHLABEL_TAG] = copy_string(match_label);            
+        }
     }
     /* Fill in the hash value of the final position reached. */
     game_details->final_hash_value = board->weak_hash_value;
@@ -1718,6 +1748,7 @@ rewrite_moves(Game *game, Board *board, Move *moves)
 {
     Boolean game_ok = TRUE;
     Move *move_details = moves;
+    unsigned plys_to_drop = GlobalState.drop_ply_number;
 
     while (game_ok && (move_details != NULL)) {
         if (*(move_details->move) != '\0') {
@@ -1741,15 +1772,6 @@ rewrite_moves(Game *game, Board *board, Move *moves)
                     move_details->evaluation = evaluate(board);
                 }
 
-#if 0
-                if (GlobalState.add_FEN_comments) {
-                    /* Append a FEN comment with the new state of the board
-                     * with the move having been played.
-                     */
-                    append_FEN_comment(move_details, board);
-                }
-                
-#endif
                 if (GlobalState.add_hashcode_comments) {
                     /* Append a hashcode comment using the new state of the board
                      * with the move having been played.
@@ -1757,29 +1779,27 @@ rewrite_moves(Game *game, Board *board, Move *moves)
                     move_details->zobrist = generate_zobrist_hash_from_board(board);
                 }
                 
+                if(GlobalState.drop_comment_pattern != NULL &&
+                        move_details->comment_list != NULL) {
+                    if(find_matching_comment(GlobalState.drop_comment_pattern,
+                                              move_details->comment_list) != NULL) {
+                        /* We have a match. */
+                        plys_to_drop = (board->move_number - 1) * 2;
+                        if(board->to_move == BLACK) {
+                            plys_to_drop++;
+                        }
+                    }
+                }
                 /* See if a comment should be replaced by a FEN comment. */
-                if(move_details->comment_list != NULL &&
-                        GlobalState.FEN_comment_pattern != NULL) {
-                    CommentList *comment = move_details->comment_list;
-                    Boolean match = FALSE;
-                    while(!match && comment != NULL) {
-                        StringList *string_list = comment->comment;
-                        while(!match && string_list != NULL) {
-                            if(strcmp(GlobalState.FEN_comment_pattern, string_list->str) == 0) {
-                                match = TRUE;
-                            }
-                            else {
-                                string_list = string_list->next;
-                            }
-                        }
-                        if(!match) {
-                            comment = comment->next;
-                        }
-                        else {
-                            /* Replace it. */
-                            (void) free((void *) string_list->str);
-                            string_list->str = get_FEN_string(board);
-                        }
+                if(GlobalState.FEN_comment_pattern != NULL &&
+                        move_details->comment_list != NULL) {
+                    StringList *comment_to_replace =
+                        find_matching_comment(GlobalState.FEN_comment_pattern,
+                                              move_details->comment_list);
+                    if(comment_to_replace != NULL) {
+                        /* Replace it. */
+                        (void) free((void *) comment_to_replace->str);
+                        comment_to_replace->str = get_FEN_string(board);
                     }
                 }
 
@@ -1810,56 +1830,61 @@ rewrite_moves(Game *game, Board *board, Move *moves)
             game_ok = FALSE;
         }
     }
-    if (!game_ok && GlobalState.keep_broken_games && move_details != NULL) {
-        /* Try to place the remaining moves into a comment. */
-        CommentList *comment = (CommentList*) malloc_or_die(sizeof (*comment));
-        /* Break the link from the previous move. */
-        Move *prev;
-        StringList *commented_move_list = NULL;
+    if (!game_ok) {
+        if(GlobalState.keep_broken_games && move_details != NULL) {
+            /* Try to place the remaining moves into a comment. */
+            CommentList *comment = (CommentList*) malloc_or_die(sizeof (*comment));
+            /* Break the link from the previous move. */
+            Move *prev;
+            StringList *commented_move_list = NULL;
 
-        /* Find the move before the current one, if any. */
-        if (move_details == moves) {
-            /* Broken from the first move. */
-            prev = NULL;
-        }
-        else {
-            prev = moves;
-            while (prev != NULL && prev->next != move_details) {
-                prev = prev->next;
+            /* Find the move before the current one, if any. */
+            if (move_details == moves) {
+                /* Broken from the first move. */
+                prev = NULL;
             }
-            if (prev != NULL) {
-                prev->next = NULL;
+            else {
+                prev = moves;
+                while (prev != NULL && prev->next != move_details) {
+                    prev = prev->next;
+                }
+                if (prev != NULL) {
+                    prev->next = NULL;
+                }
             }
-        }
-        /* Build the comment string. */
-        char *terminating_result = NULL;
-        while (move_details != NULL) {
-            commented_move_list = save_string_list_item(commented_move_list, (char *) move_details->move);
-            if (move_details->next == NULL) {
-                /* Remove the terminating result. */
-                terminating_result = move_details->terminating_result;
-                move_details->terminating_result = NULL;
+            /* Build the comment string. */
+            char *terminating_result = NULL;
+            while (move_details != NULL) {
+                commented_move_list = save_string_list_item(commented_move_list, (char *) move_details->move);
+                if (move_details->next == NULL) {
+                    /* Remove the terminating result. */
+                    terminating_result = move_details->terminating_result;
+                    move_details->terminating_result = NULL;
+                }
+                move_details = move_details->next;
             }
-            move_details = move_details->next;
-        }
-        comment->comment = commented_move_list;
-        comment->next = NULL;
+            comment->comment = commented_move_list;
+            comment->next = NULL;
 
-        if (prev != NULL) {
-            prev->terminating_result = terminating_result;
-            append_comments_to_move(prev, comment);
-        }
-        else {
-            /* The whole game is broken. */
-            if (game != NULL) {
-                if (terminating_result != NULL) {
-                    commented_move_list = save_string_list_item(commented_move_list, terminating_result);
+            if (prev != NULL) {
+                prev->terminating_result = terminating_result;
+                append_comments_to_move(prev, comment);
+            }
+            else {
+                /* The whole game is broken. */
+                if (game != NULL) {
+                    if (terminating_result != NULL) {
+                        commented_move_list = save_string_list_item(commented_move_list, terminating_result);
+                    }
+                    game->prefix_comment = comment;
+                    game->moves = NULL;
                 }
-                game->prefix_comment = comment;
-                game->moves = NULL;
             }
         }
     }
+    else if(plys_to_drop > 0 && game != NULL) {
+        game_ok = drop_plys(game, moves, plys_to_drop);
+    }
     return game_ok;
 }
 
@@ -1891,8 +1916,7 @@ rewrite_variations(const Board *board, Variation *variation)
 Board *
 rewrite_game(Game *current_game)
 {
-    const char *fen = current_game->tags[FEN_TAG];
-    Board *board = new_game_board(fen);
+    Board *board = new_game_board(current_game->tags[FEN_TAG]);
     Boolean game_ok;
 
     /* No null-move found at the start of the game. */
@@ -1908,6 +1932,72 @@ rewrite_game(Game *current_game)
     return board;
 }
 
+/* Drop the given number of play from this game.
+ * Return FALSE if there are insufficient.
+ * If there are sufficient, replace the game's FEN tag
+ * with the revised starting state.
+ */
+static Boolean
+drop_plys(Game *game, Move *moves, unsigned plys_to_drop)
+{
+    char *fen = game->tags[FEN_TAG];
+    Board *board = new_game_board(fen);
+    Boolean game_ok = TRUE;
+    unsigned plys = 0;
+    Move *move_details = moves;
+
+    if(plys_to_drop > 0 && game->prefix_comment != NULL) {
+        /* Don't free the prefix_comment because that is
+         * handled elsewhere.
+         */
+        game->prefix_comment = NULL;
+    }
+    while(plys < plys_to_drop && move_details != NULL && game_ok) {
+        /* Rewriting is not strictly necessary, because it has already been done,
+         * but it provides all the required functionality.
+         */
+        if (rewrite_move(game, board->to_move, move_details, board)) {
+            board->to_move = OPPOSITE_COLOUR(board->to_move);
+            if (board->to_move == WHITE) {
+                board->move_number++;
+            }
+            plys++;
+            move_details = move_details->next;
+        }
+        else {
+            game_ok = FALSE;
+        }
+    }
+    if(plys == plys_to_drop && move_details != NULL && game_ok) {
+        if(fen != NULL) {
+            (void) free((void *) fen);
+        }
+        // Reset the move number.
+        board->move_number = 1;
+        game->moves = move_details;        
+        game->tags[FEN_TAG] = get_FEN_string(board);
+        game->tags[SETUP_TAG] = copy_string("1");
+        if(game->tags[PLY_COUNT_TAG] != NULL || GlobalState.output_plycount) {
+            add_plycount(game);
+        }
+
+        if(game->tags[TOTAL_PLY_COUNT_TAG] != NULL || GlobalState.output_total_plycount) {
+            add_total_plycount(game, GlobalState.keep_variations);
+        }
+
+        /* Free the dropped moves. */
+        Move *move_to_drop = moves;
+        while(move_to_drop->next != move_details) {
+            move_to_drop = move_to_drop->next;
+        }
+        move_to_drop->next = NULL;
+        free_move_list(moves);
+    }
+    else {
+        game_ok = FALSE;
+    }
+    return game_ok;
+}
 
 /* Define a table to hold the positional hash codes of interest.
  * Size should be a prime number for collision avoidance.
@@ -1985,12 +2075,36 @@ Boolean using_polyglot = FALSE;
  */
 Boolean save_polyglot_hashcode(const char *value)
 {
-    uint64_t hash;
+    uint64_t hash = 0x0;
     Boolean Ok;
     
     if(value != NULL && *value != '\0') {
-        if(value[strspn(value, "0123456789abcdefABCDEF")] == '\0') {
-            Ok = sscanf(value, "%llx", &hash) == 1;
+        size_t len = strspn(value, "0123456789abcdefABCDEF");
+        if(len > 0 && value[len] == '\0') {
+            /* Avoid using %llx to convert the string as it is
+             * not converted correctly on 32-bit systems.
+             */
+            if(len <= 8) {
+                unsigned long lower;
+                Ok = sscanf(value, "%lx", &lower) == 1;
+                if(Ok) {
+                    hash = lower;
+                }
+            }
+            else {
+                char *copy = copy_string(value);
+                /* Exclude the bottom 8 characters. */
+                copy[8] = '\0';
+                /* Extract the upper 4 bytes. */
+                unsigned long upper;
+                Ok = sscanf(copy, "%lx", &upper) == 1;
+                /* Extract the lower 4 bytes. */
+                unsigned long lower;
+                Ok &= sscanf(value + 16 - len, "%lx", &lower) == 1;
+                hash = upper;
+                hash <<= 32;
+                hash |= lower;
+            }
             if (Ok) {
                 HashLog *entry = (HashLog *) malloc_or_die(sizeof (*entry));
                 unsigned ix = hash % MAX_POLYGLOT_CODE;
@@ -2005,6 +2119,9 @@ Boolean save_polyglot_hashcode(const char *value)
                 polyglot_codes_of_interest[ix] = entry;
                 using_polyglot = TRUE;
             }
+            else {
+                fprintf(GlobalState.logfile, "Unrecognised hash value %s\n", value);
+            }
         }
         else {
             Ok = FALSE;
@@ -2018,8 +2135,11 @@ Boolean save_polyglot_hashcode(const char *value)
 
 /* Does the current board match a position of interest.
  * Look in codes_of_interest for current_hash_value.
+ * Return NULL if no match, otherwise a possible label for the
+ * match to be added to the game's tags. An empty string is
+ * used for no label.
  */
-static Boolean
+static const char *
 position_matches(const Board *board)
 {
     Boolean found = FALSE;
@@ -2046,11 +2166,13 @@ position_matches(const Board *board)
             }
         }
     }
-    if (!found) {
-        const char *matching_pattern = pattern_match_board(board);
-        found = matching_pattern != NULL;
+    if (found) {
+        return "";
+    }
+    else {
+        const char *match_label = pattern_match_board(board);
+        return match_label;
     }
-    return found;
 }
 
 /* Build a basic EPD string from the given board. */
@@ -2308,7 +2430,7 @@ char *get_FEN_string(const Board *board)
 {
     char epd[FEN_SPACE], fen_suffix[FEN_SPACE];
     build_FEN_components(board, epd, fen_suffix);
-    char *FEN_string = malloc_or_die(strlen(epd) + 1 + 
+    char *FEN_string = (char *) malloc_or_die(strlen(epd) + 1 + 
             strlen(fen_suffix) + 1);
     sprintf(FEN_string, "%s %s", epd, fen_suffix);
     return FEN_string;
@@ -2365,7 +2487,7 @@ static void
 append_hashcode_comment(Move *move_details, Board *board)
 {
     uint64_t hash = generate_zobrist_hash_from_board(board);
-    char *hashcode_comment = malloc_or_die(HASH_64_BIT_SPACE + 1);
+    char *hashcode_comment = (char *) malloc_or_die(HASH_64_BIT_SPACE + 1);
     CommentList *comment = (CommentList*) malloc_or_die(sizeof (*comment));
     StringList *current_comment = save_string_list_item(NULL, hashcode_comment);
     
@@ -2517,3 +2639,34 @@ shannonEvaluation(const Board *board)
             (whiteMoveCount - blackMoveCount) * 0.1;
     return shannonValue;
 }
+
+/* Look for comment_pattern in the given list of comments.
+ * If found, return where it was found, otherwise NULL.
+ */
+static StringList *
+find_matching_comment(const char *comment_pattern,
+                      const CommentList *comment)
+{
+    Boolean match = FALSE;
+    StringList *string_list;
+    while(!match && comment != NULL) {
+        string_list = comment->comment;
+        while(!match && string_list != NULL) {
+            if(strcmp(comment_pattern, string_list->str) == 0) {
+                match = TRUE;
+            }
+            else {
+                string_list = string_list->next;
+            }
+        }
+        if(!match) {
+            comment = comment->next;
+        }
+    }
+    if(match) {
+        return string_list;
+    }
+    else {
+        return (StringList *) NULL;
+    }
+}
