@@ -1,6 +1,6 @@
 /*
  *  This file is part of pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2017 David J. Barnes
+ *  Copyright (C) 1994-2018 David J. Barnes
  *
  *  pgn-extract is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -40,7 +40,7 @@
 #include "lists.h"
 #include "mymalloc.h"
 
-#define CURRENT_VERSION "v17-55"
+#define CURRENT_VERSION "v18-10"
 #define URL "https://www.cs.kent.ac.uk/people/staff/djb/pgn-extract/"
 
 /* The prefix of the arguments allowed in an argsfile.
@@ -64,6 +64,8 @@ static void read_args_file(const char *infile);
 static game_number *extract_game_number_list(const char *number_list);
 #if defined(__unix__) || defined(__linux__) || defined(__APPLE__)
 int strcasecmp(const char *, const char *);
+#else
+int _stricmp(const char *s1, const char *s2);
 #endif
 
 /* Select the correct function according to operating system. */
@@ -73,7 +75,6 @@ stringcompare(const char *s1, const char *s2)
 #if defined(__unix__) || defined(__linux__) || defined(__APPLE__)
     return strcasecmp(s1, s2);
 #else
-    int _stricmp(const char *s1, const char *s2);
     return _stricmp(s1, s2);
 #endif
 }
@@ -185,16 +186,20 @@ usage_and_exit(void)
         "",
 
         "--addhashcode - output a HashCode tag",
+        "--addlabeltag - output a MatchLabel tag with FENPattern",
+        "--addmatchtag - output a MaterialMatch tag with -z",
         "--allownullmoves - allow NULL moves in the main line",
         "--append - see -a",
         "--checkfile - see -c",
         "--checkmate - see -M",
         "--commentlines - output each comment on a separate line",
+        "--dropbefore - drop opening ply before a matching comment string",
+        "--dropply - drop the given number of ply from the beginning of the game",
         "--duplicates - see -d",
         "--evaluation - include a position evaluation after each move",
         "--fencomments - include a FEN string after each move",
         "--fifty - only output games that include fifty moves with no capture or pawn move.",
-        "--fixresulttags - correct Result tags that conflict with the game outcome (checkmate or stalemate).",
+        "--fixresulttags - correct Result tags that conflict with the game outcome or terminating result.",
         "--fuzzydepth plies - positional duplicates match",
         "--hashcomments - include a hashcode string after each move",
         "--help - see -h",
@@ -203,6 +208,7 @@ usage_and_exit(void)
         "--linelength - see -w",
         "--matchplylimit - maximum ply depth to search for positional matches",
         "--markmatches - mark positional and material matches with a comment; see -t, -v, and -z",
+        "--nestedcomments - allow nested comments",
         "--nobadresults - reject games with inconsistent result indications.",
         "--nochecks - don't output + and # after moves.",
         "--nocomments - see -C",
@@ -243,7 +249,7 @@ usage_and_exit(void)
             "pgn-extract %s (%s): a Portable Game Notation (PGN) manipulator.\n",
             CURRENT_VERSION, __DATE__);
     fprintf(GlobalState.logfile,
-            "Copyright (C) 1994-2017 David J. Barnes (d.j.barnes@kent.ac.uk)\n");
+            "Copyright (C) 1994-2018 David J. Barnes (d.j.barnes@kent.ac.uk)\n");
     fprintf(GlobalState.logfile, "%s\n\n", URL);
     fprintf(GlobalState.logfile, "Usage: pgn-extract [arguments] [file.pgn ...]\n");
 
@@ -293,7 +299,9 @@ read_args_file(const char *infile)
                             process_roster_line(line);
                             break;
                         case ENDINGS_ARGUMENT:
-                            process_ending_line(line);
+                        case ENDINGS_COLOURED_ARGUMENT:
+                            process_ending_line(line,
+                                                linetype == ENDINGS_ARGUMENT);
                             (void) free(line);
                             break;
                         default:
@@ -389,6 +397,7 @@ read_args_file(const char *infile)
                          * the current line.
                          */
                     case ENDINGS_ARGUMENT:
+                    case ENDINGS_COLOURED_ARGUMENT:
                     case HASHCODE_MATCH_ARGUMENT:
                     case MOVES_ARGUMENT:
                     case OUTPUT_FEN_STRING_ARGUMENT:
@@ -435,6 +444,7 @@ classify_arg(const char *line)
             case MOVES_ARGUMENT:
             case POSITIONS_ARGUMENT:
             case ENDINGS_ARGUMENT:
+            case ENDINGS_COLOURED_ARGUMENT:
             case TAG_EXTRACTION_ARGUMENT:
             case LINE_WIDTH_ARGUMENT:
             case OUTPUT_FORMAT_ARGUMENT:
@@ -949,8 +959,10 @@ process_argument(char arg_letter, const char *associated_value)
             }
             break;
         case ENDINGS_ARGUMENT:
+        case ENDINGS_COLOURED_ARGUMENT:
             if (*filename != '\0') {
-                if (!build_endings(filename)) {
+                if (!build_endings(filename,
+                                   arg_letter == ENDINGS_ARGUMENT)) {
                     exit(1);
                 }
             }
@@ -985,6 +997,14 @@ process_long_form_argument(const char *argument, const char *associated_value)
         GlobalState.add_hashcode_tag = TRUE;
         return 1;
     }
+    else if (stringcompare(argument, "addlabeltag") == 0) {
+        GlobalState.add_matchlabel_tag = TRUE;
+        return 1;
+    }
+    else if (stringcompare(argument, "addmatchtag") == 0) {
+        GlobalState.add_match_tag = TRUE;
+        return 1;
+    }
     else if (stringcompare(argument, "allownullmoves") == 0) {
         GlobalState.allow_null_moves = TRUE;
         return 1;
@@ -1005,6 +1025,32 @@ process_long_form_argument(const char *argument, const char *associated_value)
         GlobalState.separate_comment_lines = TRUE;
         return 1;
     }
+    else if (stringcompare(argument, "dropbefore") == 0) {
+        /* Save the comment string to be matched. */
+        if (associated_value != NULL) {
+            GlobalState.drop_comment_pattern = copy_string(associated_value);
+        }
+        else {
+            fprintf(GlobalState.logfile,
+                    "--%s requires a string following it.\n", argument);
+            exit(1);
+        }
+        return 2;
+    }
+    else if (stringcompare(argument, "dropply") == 0) {
+        /* Extract the number. */
+        unsigned number = 0;
+
+        if (sscanf(associated_value, "%u", &number) == 1) {
+            GlobalState.drop_ply_number = number;
+        }
+        else {
+            fprintf(GlobalState.logfile,
+                    "--%s requires a positive number following it.\n", argument);
+            exit(1);
+        }
+        return 2;
+    }
     else if (stringcompare(argument, "duplicates") == 0) {
         process_argument(DUPLICATES_FILE_ARGUMENT, associated_value);
         return 2;
@@ -1115,6 +1161,10 @@ process_long_form_argument(const char *argument, const char *associated_value)
         }
         return 2;
     }
+    else if (stringcompare(argument, "nestedcomments") == 0) {
+        GlobalState.allow_nested_comments = TRUE;
+        return 1;
+    }
     else if (stringcompare(argument, "nobadresults") == 0) {
         GlobalState.reject_inconsistent_results = TRUE;
         return 1;
@@ -1387,7 +1437,7 @@ extract_game_number_list(const char *number_list)
             ok = FALSE;
         }
         if(ok) {
-            game_number *list_item = malloc_or_die(sizeof(*list_item));
+            game_number *list_item = (game_number *) malloc_or_die(sizeof(*list_item));
             list_item->min = min;
             list_item->max = max;
             list_item->next = NULL;
