@@ -1,6 +1,6 @@
 /*
  *  This file is part of pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2017 David J. Barnes
+ *  Copyright (C) 1994-2018 David J. Barnes
  *
  *  pgn-extract is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -135,13 +135,18 @@ new_move_structure(void)
     Move *move = (Move *) malloc_or_die(sizeof (Move));
 
     move->terminating_result = NULL;
+    move->piece_to_move = EMPTY;
+    move->captured_piece = EMPTY;
+    move->promoted_piece = EMPTY;
+    move->check_status = NOCHECK;
     move->epd = NULL;
     move->fen_suffix = NULL;
     move->zobrist = ~0;
     move->evaluation = 0;
-    move->Nags = NULL;
+    move->NAGs = NULL;
     move->comment_list = NULL;
     move->Variants = NULL;
+    move->prev = NULL;
     move->next = NULL;
     return move;
 }
