@@ -1,6 +1,6 @@
 /*
  *  This file is part of pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2017 David J. Barnes
+ *  Copyright (C) 1994-2018 David J. Barnes
  *
  *  pgn-extract is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -55,11 +55,12 @@ typedef enum {
  */
 
 typedef struct ending_details {
-    const char *expression;
-    /* There is not a proper distinction between black and white
-     * but we still need information on the two sets of pieces
-     * specified in the input file.
+    /* Whether the pieces are to be tried against
+     * both colours.
      */
+    Boolean both_colours;
+    const char *expression;
+    /* The number of each set of pieces. */
     int num_pieces[2][NUM_PIECE_VALUES];
     Occurs occurs[2][NUM_PIECE_VALUES];
     /* Numbers of general minor pieces. */
@@ -118,12 +119,13 @@ is_English_piece(char c)
  * in the data.
  */
 static Ending_details *
-new_ending_details(const char *expression)
+new_ending_details(const char *expression, Boolean both_colours)
 {
     Ending_details *details = (Ending_details *) malloc_or_die(sizeof (Ending_details));
     int c;
     Piece piece;
 
+    details->both_colours = both_colours;
     details->expression = copy_string(expression);
     for (piece = PAWN; piece <= KING; piece++) {
         for (c = 0; c < 2; c++) {
@@ -547,7 +549,6 @@ static Boolean
 look_for_ending(Game *game_details, Ending_details *details_to_find)
 {
     Boolean game_ok = TRUE;
-    Boolean game_matches = FALSE;
     Boolean match_comment_added = FALSE;
     Move *next_move = game_details->moves;
     Move *move_for_comment = NULL;
@@ -588,46 +589,24 @@ look_for_ending(Game *game_details, Ending_details *details_to_find)
      * moves and we haven't exceeded the search depth without finding
      * a match.
      */
-    while (game_ok && (next_move != NULL) && !game_matches) {
+    Boolean white_matches = FALSE;
+    Boolean black_matches = FALSE;
+    while (game_ok && (next_move != NULL) && !(white_matches || black_matches)) {
 	/* Try before applying each move.
 	 * Note, that we wish to try both ways around because we might
 	 * have WT,BT WF,BT ... If we don't try BLACK on WHITE success
-	 * then we might miss a match.
+	 * then we might miss a match because a full match takes several
+         * separate individual match steps.
 	 */
-	game_matches = ending_match(details_to_find, num_pieces, WHITE) |
-		ending_match(details_to_find, num_pieces, BLACK);
-        if (!game_matches) {
-            if (*(next_move->move) != '\0') {
-                if (apply_move(next_move, board)) {
-                    /* Remove any captured pieces. */
-                    if (next_move->captured_piece != EMPTY) {
-                        num_pieces[OPPOSITE_COLOUR(colour)][next_move->captured_piece]--;
-                    }
-                    if (next_move->promoted_piece != EMPTY) {
-                        num_pieces[OPPOSITE_COLOUR(colour)][next_move->promoted_piece]++;
-                        /* Remove the promoting pawn. */
-                        num_pieces[OPPOSITE_COLOUR(colour)][PAWN]--;
-                    }
-
-                    move_for_comment = next_move;
-                    colour = OPPOSITE_COLOUR(colour);
-                    next_move = next_move->next;
-                }
-                else {
-                    game_ok = FALSE;
-                }
-            }
-            else {
-                /* An empty move. */
-                fprintf(GlobalState.logfile,
-                        "Internal error: Empty move in look_for_ending.\n");
-                game_ok = FALSE;
-            }
+	white_matches = ending_match(details_to_find, num_pieces, WHITE);
+        if(details_to_find->both_colours) {
+            black_matches = ending_match(details_to_find, num_pieces, BLACK);
         }
         else {
-            /* Match.
-               See whether a matching comment is required.
-             */
+            black_matches = FALSE;
+        }
+        if (white_matches || black_matches) {
+            /* See whether a matching comment is required. */
             if (GlobalState.add_position_match_comments && !match_comment_added) {
                 CommentList *match_comment = create_match_comment(board);
                 if (move_for_comment != NULL) {
@@ -647,11 +626,45 @@ look_for_ending(Game *game_details, Ending_details *details_to_find)
                 }
             }
         }
+        else if (*(next_move->move) != '\0') {
+            /* Try the next position. */
+            if (apply_move(next_move, board)) {
+                /* Remove any captured pieces. */
+                if (next_move->captured_piece != EMPTY) {
+                    num_pieces[OPPOSITE_COLOUR(colour)][next_move->captured_piece]--;
+                }
+                if (next_move->promoted_piece != EMPTY) {
+                    num_pieces[OPPOSITE_COLOUR(colour)][next_move->promoted_piece]++;
+                    /* Remove the promoting pawn. */
+                    num_pieces[OPPOSITE_COLOUR(colour)][PAWN]--;
+                }
+
+                move_for_comment = next_move;
+                colour = OPPOSITE_COLOUR(colour);
+                next_move = next_move->next;
+            }
+            else {
+                game_ok = FALSE;
+            }
+        }
+        else {
+            /* An empty move. */
+            fprintf(GlobalState.logfile,
+                    "Internal error: Empty move in look_for_ending.\n");
+            game_ok = FALSE;
+        }
     }
-    if (!game_ok) {
-        game_matches = FALSE;
+    (void) free((void *) board);
+    if(game_ok && (white_matches || black_matches)) {
+        if(GlobalState.add_match_tag) {
+            game_details->tags[MATERIAL_MATCH_TAG] =
+                copy_string(white_matches ? "White" : "Black");
+        }
+        return TRUE;
+    }
+    else {
+        return FALSE;
     }
-    return game_matches;
 }
 
 /* Check to see whether the given moves lead to a position
@@ -673,13 +686,20 @@ check_for_ending(Game *game)
     return matches;
 }
 
+/* Decompose the text of line to extract two sets of
+ * piece configurations.
+ * If both_colours is TRUE then matches will be tried
+ * for both colours in each configuration.
+ * Otherwise, the first set of pieces are assumed to
+ * be white and the second to be black.
+ */
 Boolean
-process_ending_line(const char *line)
+process_ending_line(const char *line, Boolean both_colours)
 {
     Boolean Ok = TRUE;
 
     if (non_blank_line(line)) {
-        Ending_details *details = new_ending_details(line);
+        Ending_details *details = new_ending_details(line, both_colours);
 
         if (decompose_line(line, details)) {
             /* Add it on to the list. */
@@ -694,7 +714,7 @@ process_ending_line(const char *line)
 }
 
 Boolean
-build_endings(const char *infile)
+build_endings(const char *infile, Boolean both_colours)
 {
     FILE *fp = fopen(infile, "r");
     Boolean Ok = TRUE;
@@ -706,7 +726,7 @@ build_endings(const char *infile)
     else {
         char *line;
         while ((line = read_line(fp)) != NULL) {
-            Ok &= process_ending_line(line);
+            Ok &= process_ending_line(line, both_colours);
             (void) free(line);
         }
         (void) fclose(fp);
