@@ -1,6 +1,6 @@
 /*
  *  This file is part of pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2017 David J. Barnes
+ *  Copyright (C) 1994-2018 David J. Barnes
  *
  *  pgn-extract is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,8 +23,8 @@
 #define END_H
 
 Boolean check_for_ending(Game *game);
-Boolean build_endings(const char *infile);
-Boolean process_ending_line(const char *line);
+Boolean build_endings(const char *infile, Boolean both_colours);
+Boolean process_ending_line(const char *line, Boolean both_colours);
 
 #endif	// END_H
 
