@@ -1,6 +1,6 @@
 /*
  *  This file is part of pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2017 David J. Barnes
+ *  Copyright (C) 1994-2018 David J. Barnes
  *
  *  pgn-extract is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
 #ifndef FENMATCHER_H
 #define FENMATCHER_H
 
-void add_fen_pattern(const char *fen_pattern);
+void add_fen_pattern(const char *fen_pattern, Boolean add_reverse, const char *label);
 const char *pattern_match_board(const Board *board);
 
 #endif	// FENMATCHER_H
