@@ -1,6 +1,6 @@
 /*
  *  This file is part of pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2017 David J. Barnes
+ *  Copyright (C) 1994-2018 David J. Barnes
  *
  *  pgn-extract is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -48,6 +48,9 @@ static TokenType current_symbol = NO_TOKEN;
  */
 static unsigned RAV_level = 0;
 
+/* How often to report processing rate. */
+static unsigned PROGRESS_RATE = 1000;
+
 /* Retain details of the header of a game.
  * This comprises the Tags and any comment prefixing the
  * moves of the game.
@@ -69,7 +72,7 @@ static Move *parse_move(void);
 static Move *parse_move_unit(void);
 static CommentList *parse_opt_comment_list(void);
 Boolean parse_opt_move_number(void);
-static StringList *parse_opt_NAG_list(void);
+static void parse_opt_NAG_list(Move *move_details);
 static Variation *parse_opt_variant_list(void);
 static Variation *parse_variant(void);
 static char *parse_result(void);
@@ -77,7 +80,6 @@ static char *parse_result(void);
 static void setup_for_new_game(void);
 void free_tags(void);
 static void check_result(char **Tags, const char *terminating_result);
-static void free_comment_list(CommentList *comment_list);
 static void deal_with_ECO_line(Move *move_list);
 static void deal_with_game(Move *move_list);
 static Boolean finished_processing(void);
@@ -301,6 +303,9 @@ parse_opt_game_list(SourceFileType file_type)
         move_list = NULL;
         setup_for_new_game();
     }
+    if(move_list != NULL) {
+        free_move_list(move_list);
+    }
 }
 
 /* Parse a game and return a pointer to any valid list of moves
@@ -551,13 +556,10 @@ parse_move(void)
     /* @@@ Watch out for finding just the number. */
     move_details = parse_move_unit();
     if (move_details != NULL) {
-        CommentList *comment;
-
-        move_details->Nags = parse_opt_NAG_list();
-        comment = parse_opt_comment_list();
-        if (comment != NULL) {
-            move_details->comment_list = append_comment(comment, move_details->comment_list);
-        }
+        parse_opt_NAG_list(move_details);
+        /* Any trailing comments will have been picked up
+         * and attached to the NAGs.
+         */
     }
     return move_details;
 }
@@ -571,8 +573,10 @@ parse_move_unit(void)
         move_details = yylval.move_details;
 
         if (move_details->class == NULL_MOVE && RAV_level == 0) {
-            print_error_context(GlobalState.logfile);
-            fprintf(GlobalState.logfile, "Null moves (--) only allowed in variations.\n");
+            if(!GlobalState.allow_null_moves) {
+                print_error_context(GlobalState.logfile);
+                fprintf(GlobalState.logfile, "Null moves (--) only allowed in variations.\n");
+            }
         }
 
         current_symbol = next_token();
@@ -619,21 +623,34 @@ parse_opt_move_number(void)
     return something_found;
 }
 
-static StringList *
-parse_opt_NAG_list(void)
+/**
+ * Parse 0 or more NAGs, optionally followed by 0 or more comments.
+ * @param move_details
+ */
+static void
+parse_opt_NAG_list(Move *move_details)
 {
-    StringList *nags = NULL;
-
     while (current_symbol == NAG) {
-        if (GlobalState.keep_NAGs) {
-            nags = save_string_list_item(nags, yylval.token_string);
+        Nag *details = (Nag *) malloc_or_die(sizeof(*details));
+        details->text = NULL;
+        details->comments = NULL;
+        details->next = NULL;
+        do {
+            details->text = save_string_list_item(details->text, yylval.token_string);
+            current_symbol = next_token();
+        } while(current_symbol == NAG);
+        details->comments = parse_opt_comment_list();
+        if(move_details->NAGs == NULL) {
+            move_details->NAGs = details;
         }
         else {
-            (void) free((void *) yylval.token_string);
+            Nag *nextNAG = move_details->NAGs;
+            while(nextNAG->next != NULL) {
+                nextNAG = nextNAG->next;
+            }
+            nextNAG->next = details;
         }
-        current_symbol = next_token();
     }
-    return nags;
 }
 
 static Variation *
@@ -666,7 +683,7 @@ parse_variant(void)
         Move *moves;
 
         RAV_level++;
-        variation = malloc_or_die(sizeof (Variation));
+        variation = (Variation *) malloc_or_die(sizeof (Variation));
 
         current_symbol = next_token();
         prefix_comment = parse_opt_comment_list();
@@ -774,7 +791,7 @@ free_string_list(StringList *list)
     }
 }
 
-static void
+void
 free_comment_list(CommentList *comment_list)
 {
     while (comment_list != NULL) {
@@ -803,40 +820,48 @@ free_variation(Variation *variation)
             free_comment_list(next->suffix_comment);
         }
         if (next->moves != NULL) {
-            (void) free_move_list((void *) next->moves);
+            (void) free_move_list(next->moves);
         }
         (void) free((void *) next);
     }
 }
 
+static void free_NAG_list(Nag *nag_list)
+{
+    while(nag_list != NULL) {
+        Nag *nextNAG = nag_list->next;
+        free_string_list(nag_list->text);
+        free_comment_list(nag_list->comments);
+        (void) free((void *) nag_list);
+        nag_list = nextNAG;
+    }
+}
+
 void
 free_move_list(Move *move_list)
 {
-    Move *next;
+    Move *nextMove;
 
     while (move_list != NULL) {
-        next = move_list;
+        nextMove = move_list;
         move_list = move_list->next;
-        if (next->Nags != NULL) {
-            free_string_list(next->Nags);
+        
+        free_NAG_list(nextMove->NAGs);
+        free_comment_list(nextMove->comment_list);
+        free_variation(nextMove->Variants);
+        
+        if (nextMove->epd != NULL) {
+            (void) free((void *) nextMove->epd);
         }
-        if (next->comment_list != NULL) {
-            free_comment_list(next->comment_list);
+        if(nextMove->fen_suffix != NULL) {
+            (void) free((void *) nextMove->fen_suffix);
+            nextMove->fen_suffix = NULL;
         }
-        if (next->Variants != NULL) {
-            free_variation(next->Variants);
+        if (nextMove->terminating_result != NULL) {
+            (void) free((void *) nextMove->terminating_result);
         }
-        if (next->epd != NULL) {
-            (void) free((void *) next->epd);
-        }
-        if(next->fen_suffix != NULL) {
-            (void) free((void *) next->fen_suffix);
-            next->fen_suffix = NULL;
-        }
-        if (next->terminating_result != NULL) {
-            (void) free((void *) next->terminating_result);
-        }
-        (void) free((void *) next);
+        
+        (void) free((void *) nextMove);
     }
 }
 
@@ -864,6 +889,16 @@ save_string_list_item(StringList *list, const char *str)
             tail->next = new_item;
         }
     }
+#if 1
+    /* This is almost certainly correct to avoid losing
+     * two bytes with malloc'd empty strings.
+     * No problems with valgrind but just being
+     * cautious.
+     */
+    else if(str != NULL) {
+        (void) free((void *) str);
+    }
+#endif
     return list;
 }
 
@@ -1064,20 +1099,24 @@ deal_with_game(Move *move_list)
                     outputfile = GlobalState.duplicate_file;
                     if ((last_input_file != GlobalState.current_input_file) &&
                             (GlobalState.current_input_file != NULL)) {
-                        /* Record which file this and succeeding
-                         * duplicates come from.
-                         */
-                        print_str(outputfile, "{ From: ");
-                        print_str(outputfile,
-                                GlobalState.current_input_file);
-                        print_str(outputfile, " }");
-                        terminate_line(outputfile);
+                        if(GlobalState.keep_comments) {
+                            /* Record which file this and succeeding
+                             * duplicates come from.
+                             */
+                            print_str(outputfile, "{ From: ");
+                            print_str(outputfile,
+                                    GlobalState.current_input_file);
+                            print_str(outputfile, " }");
+                            terminate_line(outputfile);
+                        }
                         last_input_file = GlobalState.current_input_file;
                     }
-                    print_str(outputfile, "{ First found in: ");
-                    print_str(outputfile, original_filename);
-                    print_str(outputfile, " }");
-                    terminate_line(outputfile);
+                    if(GlobalState.keep_comments) {
+                        print_str(outputfile, "{ First found in: ");
+                        print_str(outputfile, original_filename);
+                        print_str(outputfile, " }");
+                        terminate_line(outputfile);
+                    }
                 }
                 /* Now output what we have. */
                 output_game(&current_game, outputfile);
@@ -1124,7 +1163,7 @@ deal_with_game(Move *move_list)
         free_position_count_list(current_game.position_counts);
         current_game.position_counts = NULL;
     }
-    if (GlobalState.verbosity != 0 && (GlobalState.num_games_processed % 10) == 0) {
+    if (GlobalState.verbosity != 0 && (GlobalState.num_games_processed % PROGRESS_RATE) == 0) {
         fprintf(stderr, "Games: %lu\r", GlobalState.num_games_processed);
     }
 }
