@@ -1,6 +1,6 @@
 /*
  *  This file is part of pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2017 David J. Barnes
+ *  Copyright (C) 1994-2018 David J. Barnes
  *
  *  pgn-extract is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -337,6 +337,10 @@ previous_virtual_occurance(Game game_details)
             /* Where to write the next VirtualHashLog entry. */
             static long next_free_entry = 0l;
 
+            /* Avoid valgrind error when writing unset bytes that
+             * are part of the structure padding.
+             */
+            memset((void *) &entry, 0, sizeof(entry));
             /* Store the XOR of the two hash values. */
             entry.final_hash_value = game_details.final_hash_value;
             entry.cumulative_hash_value = game_details.cumulative_hash_value;
