@@ -1,6 +1,6 @@
 /*
  *  This file is part of pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2017 David J. Barnes
+ *  Copyright (C) 1994-2018 David J. Barnes
  *
  *  pgn-extract is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -140,7 +140,7 @@ add_to_taglist(const char *str, StringArray *list)
             }
         }
         else {
-            list->tag_strings = (TagSelection *) realloc((void *) list->tag_strings,
+            list->tag_strings = (TagSelection *) realloc_or_die((void *) list->tag_strings,
                     (list->num_allocated_elements + MORE_LIST_SPACE + 1) *
                     sizeof (TagSelection));
             if (list->tag_strings != NULL) {
@@ -288,7 +288,7 @@ add_tag_to_list(int tag, const char *tagstr, TagOperator operator)
         extend_tag_list_length(tag + 1);
     }
     if(tag == FEN_TAG) {
-        add_fen_pattern_match(tagstr);
+        add_fen_pattern_match(tagstr, FALSE, NULL);
     }
     else if ((tag >= 0) && (tag < tag_list_length)) {
         const char *string_to_store = tagstr;
@@ -462,14 +462,16 @@ check_date(const char *date_string, StringArray *list)
     return wanted;
 }
 
+/* Check whether the Elo value matches any of those
+ * in the list.
+ */
 static Boolean
 check_elo(const char *elo_string, StringArray *list)
 {
     unsigned list_index;
-    Boolean wanted = TRUE;
+    Boolean wanted = FALSE;
 
-    for (list_index = 0; (list_index < list->num_used_elements) && wanted;
-            list_index++) {
+    for (list_index = 0; (list_index < list->num_used_elements) && !wanted; list_index++) {
         const char *list_string = list->tag_strings[list_index].tag_string;
         TagOperator operator = list->tag_strings[list_index].operator;
 
@@ -605,12 +607,13 @@ check_tag_details_not_ECO(char *Details[], int num_details)
                 wanted = FALSE;
             }
         }
-        else if (TagLists[PSEUDO_ELO_TAG].num_used_elements != 0) {
+
+        if (TagLists[PSEUDO_ELO_TAG].num_used_elements != 0) {
             /* Check both the White and Black lists. */
             if (Details[WHITE_ELO_TAG] != NULL) {
                 wanted = check_elo(Details[WHITE_ELO_TAG],
                         &TagLists[PSEUDO_ELO_TAG]);
-                /* If we didn't find a player there, try for the opponent. */
+                /* If we didn't find the ELO there, try for the opponent. */
                 if (!wanted && (Details[BLACK_ELO_TAG] != NULL)) {
                     wanted = check_elo(Details[BLACK_ELO_TAG],
                             &TagLists[PSEUDO_ELO_TAG]);
