@@ -1,6 +1,6 @@
 /*
  *  This file is part of pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2017 David J. Barnes
+ *  Copyright (C) 1994-2018 David J. Barnes
  *
  *  pgn-extract is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -104,6 +104,7 @@ StateInfo GlobalState = {
     -1,                 /* output_ply_limit (--plylimit) */
     0,                  /* stability_threshold (--stable) */
     0,                  /* maximum_matches */
+    0,                  /* drop_ply_number */
     FALSE,              /* output_FEN_string */
     FALSE,              /* add_FEN_comments (--fencomments) */
     FALSE,              /* add_hashcode_comments (--hashcomments) */
@@ -116,11 +117,15 @@ StateInfo GlobalState = {
     FALSE,              /* split_variants (--separatevariants) */
     FALSE,              /* reject_inconsistent_results (--nobadresults) */
     FALSE,              /* allow_null_moves (--allownullmoves) */
+    FALSE,              /* allow_nested_comments (--nestedcomments) */
+    FALSE,              /* add_match_tag (--addmatchtag) */
+    FALSE,              /* add_matchlabel_tag (--addlabeltag) */
     0,                  /* split_depth_limit */
     NORMALFILE,         /* current_file_type */
     SETUP_TAG_OK,       /* setup_status */
     "MATCH",            /* position_match_comment (--markpositionmatches) */
     (char *) NULL,      /* FEN_comment_pattern (-Fpattern) */
+    (char *) NULL,      /* drop_comment_pattern (--dropbefore) */
     (char *) NULL,      /* current_input_file */
     DEFAULT_ECO_FILE,   /* eco_file (-e) */
     (FILE *) NULL,      /* outputfile (-o, -a). Default is stdout */
@@ -313,11 +318,13 @@ main(int argc, char *argv[])
                 case MOVES_ARGUMENT:
                 case POSITIONS_ARGUMENT:
                 case ENDINGS_ARGUMENT:
-                { /* From the command line, we require an
-                         * associated file argument.
-                         * Check this here, as it is not the case
-                         * when reading arguments from an argument file.
-                         */
+                case ENDINGS_COLOURED_ARGUMENT:
+                { 
+                    /* From the command line, we require an
+                     * associated file argument.
+                     * Check this here, as it is not the case
+                     * when reading arguments from an argument file.
+                     */
                     const char *filename = &(argument[2]);
                     const char argument_letter = argument[1];
                     if (*filename == '\0') {
