@@ -1,6 +1,6 @@
 /*
  *  This file is part of pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2017 David J. Barnes
+ *  Copyright (C) 1994-2018 David J. Barnes
  *
  *  pgn-extract is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -1910,14 +1910,14 @@ determine_move_details(Colour colour, Move *move_details, Board *board)
                 /* We failed to find the move, for some reason. */
                 /* See if it might be a Bishop move with a lower case 'b'. */
                 if (move_details->move[0] == 'b') {
-                    /* See if we can find an valid Bishop alternative for it. */
-                    unsigned char alternative_move[MAX_MOVE_LEN + 1];
-                    Move *alternative;
-
+                    /* See if we can find a valid Bishop alternative for it. */
+                    unsigned char *alternative_move =
+                           (unsigned char *) malloc_or_die(strlen((char *) move_details->move) + 1);
                     strcpy((char *) alternative_move,
                             (const char *) move_details->move);
                     *alternative_move = 'B';
-                    alternative = decode_move((unsigned char *) alternative_move);
+                    Move *alternative = decode_move((unsigned char *) alternative_move);
+                    (void) free((void *) alternative_move);
                     if (alternative != NULL) {
                         Ok = bishop_move(alternative, colour, board);
                         if (Ok) {
