@@ -1,6 +1,6 @@
 /*
  *  This file is part of pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2017 David J. Barnes
+ *  Copyright (C) 1994-2018 David J. Barnes
  *
  *  pgn-extract is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -175,7 +175,7 @@ compose_variation(char *line)
         else {
             /* See if we need some more space. */
             if (num_moves == max_moves) {
-                move_list = (variant_move *) realloc((void *) move_list,
+                move_list = (variant_move *) realloc_or_die((void *) move_list,
                         (max_moves + MOVE_INCREMENT) * sizeof (*move_list));
                 if (move_list == NULL) {
                     /* Catastrophic failure. */
@@ -369,9 +369,9 @@ add_fen_positional_match(const char *fen_string)
 /* Treat fen_pattern as being a position to be matched.
  */
 void
-add_fen_pattern_match(const char *fen_pattern)
+add_fen_pattern_match(const char *fen_pattern, Boolean add_reverse, const char *label)
 {
-    add_fen_pattern(fen_pattern);
+    add_fen_pattern(fen_pattern, add_reverse, label);
     GlobalState.positional_variations = TRUE;
 }
 
