@@ -1,6 +1,6 @@
 /*
  *  This file is part of pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2017 David J. Barnes
+ *  Copyright (C) 1994-2018 David J. Barnes
  *
  *  pgn-extract is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -74,7 +74,6 @@ static void print_EPD_move_list(Game *current_game, FILE *outputfile,
         unsigned move_number, Boolean white_to_move,
         Board *final_board);
 static const char *build_FEN_comment(const Board *board);
-static void add_total_plycount(const Game *game, Boolean count_variations);
 static void add_hashcode_tag(const Game *game);
 static unsigned count_single_move_ply(const Move *move_details, Boolean count_variations);
 static unsigned count_move_list_ply(Move *move_list, Boolean count_variations);
@@ -114,6 +113,7 @@ static int DefaultTagOrder[] = {
     SETUP_TAG,
     FEN_TAG,
     TERMINATION_TAG, MODE_TAG, PLY_COUNT_TAG,
+    MATERIAL_MATCH_TAG,
 #endif
     /* The final value should be negative. */
     -1
@@ -226,7 +226,10 @@ select_tag_string(TagName tag)
 {
     const char *tag_string;
 
-    if ((tag == PSEUDO_PLAYER_TAG) || (tag == PSEUDO_ELO_TAG) || (tag == PSEUDO_FEN_PATTERN_TAG)) {
+    if ((tag == PSEUDO_PLAYER_TAG) || 
+            (tag == PSEUDO_ELO_TAG) || 
+            (tag == PSEUDO_FEN_PATTERN_TAG) ||
+            (tag == PSEUDO_FEN_PATTERN_I_TAG)) {
         tag_string = NULL;
     }
     else {
@@ -332,12 +335,6 @@ show_tags(FILE *outfp, char **Tags, int tags_length)
             output_tag(tag, copy_of_tags, outfp);
             copy_of_tags[tag] = (char *) NULL;
         }
-        /* Handle the extra tags. */
-        for (tag_index = 0; tag_index < tags_length; tag_index++) {
-            if (copy_of_tags[tag_index] != NULL) {
-                output_tag(tag_index, copy_of_tags, outfp);
-            }
-        }
     }
     else {
         for (tag_index = 0; TagOrder[tag_index] >= 0; tag_index++) {
@@ -346,6 +343,12 @@ show_tags(FILE *outfp, char **Tags, int tags_length)
             copy_of_tags[tag] = (char *) NULL;
         }
     }
+    /* Handle the remaining tags. */
+    for (tag_index = 0; tag_index < tags_length; tag_index++) {
+        if (copy_of_tags[tag_index] != NULL) {
+            output_tag(tag_index, copy_of_tags, outfp);
+        }
+    }
     (void) free(copy_of_tags);
     putc('\n', outfp);
 }
@@ -851,37 +854,47 @@ print_items_following_move(FILE *outputfile, const Move *move_details,
         unsigned move_number, Boolean white_to_move)
 {
     Boolean something_printed = FALSE;
-            StringList *nags = move_details->Nags;
-            Variation *variants = move_details->Variants;
-    if (GlobalState.keep_NAGs) {
-        if(GlobalState.json_format && move_details->Nags != NULL) {
+    Nag *nags = move_details->NAGs;
+    Variation *variants = move_details->Variants;
+    if (move_details->comment_list != NULL && GlobalState.keep_comments) {
+        print_comment_list(outputfile, move_details->comment_list);
+        something_printed = TRUE;
+    }
+    if(nags != NULL) {
+        /* We don't need to output move numbers after just
+         * NAGs, so don't set something_printed just for NAGs.
+         */
+        if (GlobalState.keep_NAGs && GlobalState.json_format) {
             fputs(", \"nags\" : [", outputfile);
         }
         while (nags != NULL) {
-            if(GlobalState.json_format) {
-                fprintf(outputfile, "\"%s\"", nags->str);
-                if(nags->next != NULL) {
-                    fputs(", ", outputfile);
+            if(GlobalState.keep_NAGs) {
+                StringList *text = nags->text;
+                while(text != NULL) {
+                    if(GlobalState.json_format) {
+                        fprintf(outputfile, "\"%s\"", text->str);
+                        if(nags->next != NULL) {
+                            fputs(", ", outputfile);
+                        }
+                    }
+                    else {
+                        print_separator(outputfile);
+                        print_str(outputfile, text->str);
+                    }
+                    text = text->next;
                 }
             }
-            else {
-                print_separator(outputfile);
-                print_str(outputfile, nags->str);
+            if(nags->comments != NULL && GlobalState.keep_comments) {
+                // @@@ JSON option needed.
+                
+                print_comment_list(outputfile, nags->comments);
+                something_printed = TRUE;
             }
             nags = nags->next;
         }
-        if(GlobalState.json_format && move_details->Nags != NULL) {
+        if(GlobalState.keep_NAGs && GlobalState.json_format) {
             fputs("] ", outputfile);
         }
-        /* We don't need to output move numbers after just
-         * NAGs, so don't set something_printed.
-         */
-    }
-    if (GlobalState.keep_comments) {
-        if (move_details->comment_list != NULL) {
-            print_comment_list(outputfile, move_details->comment_list);
-            something_printed = TRUE;
-        }
     }
     if (GlobalState.output_evaluation) {
         if(GlobalState.json_format) {
@@ -926,7 +939,7 @@ print_items_following_move(FILE *outputfile, const Move *move_details,
                     move_details->zobrist);
         }
         else {
-            char *hashcode = malloc_or_die(HASH_64_BIT_SPACE + 1);
+            char *hashcode = (char *) malloc_or_die(HASH_64_BIT_SPACE + 1);
             sprintf(hashcode, "%llx", move_details->zobrist);
             
             print_as_comment(outputfile, hashcode);
@@ -1210,10 +1223,19 @@ format_game(Game *current_game, FILE *outputfile)
 {
     Boolean white_to_move = TRUE;
     unsigned move_number = 1;
-    Board *initial_board = new_game_board(current_game->tags[FEN_TAG]);
+    Board *initial_board;
     /* The final board position, if available. */
     Board *final_board = NULL;
 
+    /* We need a copy of the final board.
+     * Combine the generation of this with a rewrite
+     * of the moves of the game into
+     * SAN (Standard Algebraic Notation) unless the original
+     * source form is required.
+     */
+    final_board = rewrite_game(current_game);
+    initial_board = new_game_board(current_game->tags[FEN_TAG]);
+
     /* If we aren't starting from the initial setup, then we
      * need to know the current move number and whose
      * move it is.
@@ -1226,22 +1248,9 @@ format_game(Game *current_game, FILE *outputfile)
     /* Start at the beginning of a line. */
     line_length = 0;
     
-    /* We need a copy of the final board.
-     * Combine the generation of this with a rewrite
-     * of the moves of the game into
-     * SAN (Standard Algebraic Notation) unless the original
-     * source form is required.
-     */
-    final_board = rewrite_game(current_game);
     if (final_board != NULL) {
         if (GlobalState.output_plycount) {
-            char count[FORMATTED_NUMBER_SIZE];
-            unsigned ply = 2 * (final_board->move_number - 1);
-            if(final_board->to_move == BLACK) {
-                ply++;
-            }
-            sprintf(count, "%u", ply);
-            current_game->tags[PLY_COUNT_TAG] = copy_string(count);
+            add_plycount(current_game);
         }
         if (GlobalState.output_total_plycount) {
             add_total_plycount(current_game, GlobalState.keep_variations);
@@ -1540,7 +1549,7 @@ build_FEN_comment(const Board *board)
     const char *prefix = "{ \"";
     const char *suffix = "\" }";
     
-    char *comment = malloc_or_die(strlen(prefix) + strlen(fen) + strlen(suffix) + 1);
+    char *comment = (char *) malloc_or_die(strlen(prefix) + strlen(fen) + strlen(suffix) + 1);
     sprintf(comment, "%s%s%s", prefix, fen, suffix);
     (void) free((void *) fen);
     return comment;
@@ -1582,7 +1591,23 @@ static unsigned count_move_list_ply(Move *move_list, Boolean count_variations)
  * Count how many plies in the game in total.
  * Include variations if count_variations.
  */
-static void add_total_plycount(const Game *game, Boolean count_variations)
+void add_plycount(const Game *game)
+{
+    unsigned count = count_move_list_ply(game->moves, FALSE);
+    char formatted_count[FORMATTED_NUMBER_SIZE];
+    sprintf(formatted_count, "%u", count);
+
+    if (game->tags[PLY_COUNT_TAG] != NULL) {
+        (void) free(game->tags[PLY_COUNT_TAG]);
+    }
+    game->tags[PLY_COUNT_TAG] = copy_string(formatted_count);
+}
+
+/*
+ * Count how many plies in the game in total.
+ * Include variations if count_variations.
+ */
+void add_total_plycount(const Game *game, Boolean count_variations)
 {
     unsigned count = count_move_list_ply(game->moves, count_variations);
     char formatted_count[FORMATTED_NUMBER_SIZE];
