@@ -1,6 +1,6 @@
 /*
  *  This file is part of pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2017 David J. Barnes
+ *  Copyright (C) 1994-2018 David J. Barnes
  *
  *  pgn-extract is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,7 +29,8 @@ OutputFormat which_output_format(const char *arg);
 const char *output_file_suffix(OutputFormat format);
 void add_to_output_tag_order(TagName tag);
 void set_output_line_length(unsigned max);
-
+void add_plycount(const Game *game);
+void add_total_plycount(const Game *game, Boolean count_variations);
 /* Provide enough static space to build FEN string. */
 #define FEN_SPACE 100
 
