@@ -1,6 +1,6 @@
 /*
  *  This file is part of pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2017 David J. Barnes
+ *  Copyright (C) 1994-2018 David J. Barnes
  *
  *  pgn-extract is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -117,7 +117,7 @@ process_tag_line(const char *TagFile, char *line)
             /* Allow for an optional operator. */
             TagOperator operator = NONE;
 
-            /* Skip whitespace up to a double quote. */
+            /* Skip whitespace. */
             while (is_character_class(*linep, WHITESPACE)) {
                 linep++;
             }
@@ -159,7 +159,7 @@ process_tag_line(const char *TagFile, char *line)
                         linep++;
                         break;
                 }
-                /* Skip whitespace up to a double quote. */
+                /* Skip whitespace. */
                 while (is_character_class(*linep, WHITESPACE)) {
                     linep++;
                 }
@@ -172,7 +172,7 @@ process_tag_line(const char *TagFile, char *line)
                 line = resulting_line.line;
                 linep = resulting_line.linep;
                 if (tag_token == TAG) {
-                    /* Treat FEN* tags as a special case.
+                    /* Treat FEN and FENPattern tags as special cases.
                      * Use the position they represent to indicate
                      * a positional match.
                      */
@@ -180,8 +180,25 @@ process_tag_line(const char *TagFile, char *line)
                         add_fen_positional_match(yylval.token_string);
                         (void) free((void *) yylval.token_string);
                     }
-                    else if (tag_index == PSEUDO_FEN_PATTERN_TAG) {
-                        add_fen_pattern_match(yylval.token_string);
+                    else if (tag_index == PSEUDO_FEN_PATTERN_TAG ||
+                            tag_index == PSEUDO_FEN_PATTERN_I_TAG) {
+                        /* Skip whitespace. */
+                        while (is_character_class(*linep, WHITESPACE)) {
+                            linep++;
+                        }
+                        const char *label;
+                        if(*linep != '\0') {
+                            /* Treat the remainder of the line as a label. */
+                            label = (const char *) linep;
+                        }
+                        else {
+                            label = NULL;
+                        }
+                        /* Generate an inverted version as well if
+                         * it is PSEUDO_FEN_PATTERN_I_TAG.
+                         */
+                        add_fen_pattern_match(yylval.token_string, 
+                                tag_index == PSEUDO_FEN_PATTERN_I_TAG, label);
                         (void) free((void *) yylval.token_string);
                     }
                     else {
