@@ -1,6 +1,6 @@
 /*
  *  This file is part of pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2017 David J. Barnes
+ *  Copyright (C) 1994-2018 David J. Barnes
  *
  *  pgn-extract is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -51,12 +51,25 @@ typedef enum {
     EVENT_DATE_TAG,
     EVENT_SPONSOR_TAG,
     FEN_TAG,
-    /* The PSEUDO_FEN_PATTERN_TAG is not a real PGN one.  It is used with the -t
+    /* The PSEUDO_FEN_PATTERN_TAGs are not real PGN ones.  They are used with the -t
      * argument so that it becomes possible to indicate a FEN-based board pattern.
+     * The _I version indicates that the pattern should be tried in both its
+     * written form and inverted for the opposite colour.
      */
     PSEUDO_FEN_PATTERN_TAG,
+    PSEUDO_FEN_PATTERN_I_TAG,
     HASHCODE_TAG,
     LONG_ECO_TAG,
+    /* The MATCHLABEL_TAG is not a real PGN one.  It is used with the -t
+     * argument and FENPattern pseudo tag so that it becomes possible to
+     * which FENPattern has been matched in a game.
+     */
+    MATCHLABEL_TAG,
+    /* The MATERIAL_MATCH_TAG is not a real PGN one.  It is used with the -z
+     * argument so that it becomes possible to indicate which player's material
+     * matches the first material pattern in a match.
+     */
+    MATERIAL_MATCH_TAG,
     MODE_TAG,
     NIC_TAG,
     OPENING_TAG,
