@@ -1,6 +1,6 @@
 /*
  *  Program: pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2017 David Barnes
+ *  Copyright (C) 1994-2018 David Barnes
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 1, or (at your option)
@@ -56,12 +56,12 @@ typedef struct string_list {
     struct string_list *next;
 } StringList;
 
-typedef struct comment_list{
+typedef struct comment_list {
     StringList *comment;
     struct comment_list *next;
 } CommentList;
 
-typedef struct variation{
+typedef struct variation {
     CommentList *prefix_comment;
     struct move *moves;
     /* The role of a suffix_comment is not entirely clear.
@@ -77,6 +77,12 @@ typedef struct variation{
     struct variation *next;
 } Variation;
 
+typedef struct nag {
+    StringList *text;
+    CommentList *comments;
+    struct nag *next; 
+} Nag;
+
 /* Define a maximum length for the text of moves.
  * This is generous.
  */
@@ -122,7 +128,7 @@ typedef struct move {
      * engine, say.
      */
     double evaluation;
-    StringList *Nags;
+    Nag *NAGs;
     CommentList *comment_list;
     /* terminating_result holds the result of the current list of moves. */
     char *terminating_result;
@@ -251,12 +257,14 @@ typedef struct {
     Boolean add_ECO;
     /* Whether an ECO file is currently being parsed. */
     Boolean parsing_ECO_file;
+    
     /* Which level to divide the output. */
     EcoDivision ECO_level;
     /* What form to write the output in. */
     OutputFormat output_format;
     /* Maximum output line length. */
     unsigned max_line_length;
+    
     /* Whether to use a virtual hash table or not. */
     Boolean use_virtual_hash_table;
     /* Whether to match on the number of moves in a game. */
@@ -287,6 +295,7 @@ typedef struct {
     Boolean tag_match_anywhere;
     /* Whether to match only games involving underpromotion. */
     Boolean match_underpromotion;
+    
     /* Maximum ply depth to search for positional variations (-x).
      * This is picked up from the length of variations in the positional
      * variations file.
@@ -311,6 +320,9 @@ typedef struct {
     unsigned quiescence_threshold;
     /* Maximum number to output (maximum_matches > 0) */
     unsigned long maximum_matches;
+    /* Number of ply to drop at the start of the game. */
+    unsigned drop_ply_number;
+    
     /* Whether to output a FEN string. Either at the end of the game
      * or replacing a matching comment (see FEN_comment_pattern). */
     Boolean output_FEN_string;
@@ -336,6 +348,13 @@ typedef struct {
     Boolean reject_inconsistent_results;
     /* Whether to allow NULL moves in the main line. */
     Boolean allow_null_moves;
+    /* Whether to allow nested comments. */
+    Boolean allow_nested_comments;
+    /* Whether to add a MaterialMatch tag with -z. */
+    Boolean add_match_tag;
+    /* Whether to add a MatchLabel tag with FENPattern */
+    Boolean add_matchlabel_tag;
+    
     /* The depth limit for splitting variations.
      * 0 => no limit.
      */
@@ -348,6 +367,8 @@ typedef struct {
     const char *position_match_comment;
     /* The comment pattern to match for FEN comments (see output_FEN_string) */
     const char *FEN_comment_pattern;
+    /* The comment pattern to match for dropping ply */
+    const char *drop_comment_pattern;
     /* Current input file name. */
     const char *current_input_file;
     /* File of ECO lines. */
